import { HttpCRUDTranslator } from '@/http-crud-translator/http-crud-translator';
import { Utilities } from '@/util/Utilities';
import {
    DateRange, Form, LoadingService, Modal, ModalService, Select,
} from '@zeedhi/common';
import {
    IEventParam, Metadata,
} from '@zeedhi/core';
import { Calendar } from '@zeedhi/zd-calendar';

export class CalendarioController {
    public modalFiltrar!: Modal;

    public filtroHabilitado!: Boolean;

    public async criarModais() {
        this.modalFiltrar = await ModalService.createFromJson('fm_00000_000_calendario/modal/fm_00000_000_filtrar', true, true);
    }

    public destruirModais() {
        ModalService.delete(this.modalFiltrar);
    }

    public abrirModalFiltrar() {
        this.modalFiltrar.show();
    }

    public fecharModalFiltrar() {
        this.modalFiltrar.hide();
    }

    public getDados({ component }: any) {
        const dataHoje = new Date();
        const dataInicial = new Date(dataHoje.getFullYear(), dataHoje.getMonth(), 1);
        const dataFinal = new Date(dataHoje.getFullYear(), dataHoje.getMonth() + 1, 0);

        this.postTarefasCalendario({ component }, dataInicial.toLocaleDateString(), dataFinal.toLocaleDateString(), null, null, null, null, null, null);
    }

    public postTarefasCalendario({ component }: IEventParam<Calendar>, dataInicial: any, dataFinal: any, local: any, localSuperior: any, tipoLocal: any, operador: any, equipes: any, cliente: any) {
        let calendarEvent: any = {};
        component.clearCalendarEvents();
        LoadingService.show();
        HttpCRUDTranslator.post('/getTarefasCalendario', {
            PERIODOINICIAL: dataInicial,
            PERIODOFINAL: dataFinal,
            NRLOCALCLIENTE: local,
            NRLOCALCLIENTEPAI: localSuperior,
            NRTIPOLOCALCLI: tipoLocal,
            NREQUIPE: equipes,
            CDOPERADOR: operador,
            CDCLIENTE: cliente,
        }).then((response: any) => {
            response.data.data.forEach((element: any) => {
                let responsavel;

                if (element.NMOPERADOR) {
                    responsavel = element.NMOPERADOR;
                } else if (element.NMEQUIPE) {
                    responsavel = element.NMEQUIPE;
                } else {
                    responsavel = 'Sem responsável definido';
                }

                calendarEvent = {
                    id: element.CODIGO,
                    name: element.NOME,
                    start: this.formatarData(element.DTHORAAGEND),
                    end: this.formatarData(element.DTHRFINALAGEND),
                    description: `Cliente: ${element.CLIENTE} <br> Local: ${element.NMLOCALCLIENTE} <br> Responsável: ${responsavel} <br> Horário: ${element.DTHORAAGEND.split(' ')[1]} - ${element.DTHRFINALAGEND.split(' ')[1]} <br> Situação: ${element.SITUACAO}`,
                    color: element.DSCORIDLOCAL,
                };
                component.addCalendarEvents([calendarEvent]);
            });
        }).catch((error: any) => {
            console.warn(error);
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            this.fecharModalFiltrar();
        });
    }

    public limparFiltros() {
        const calendario = Metadata.getInstance<Calendar>('calendarioFmapp');
        Metadata.getInstance<Form>('form-filtros').clearForm();
        this.getDados({ component: calendario });
        this.filtroHabilitado = false;
    }

    public formatarData(data: any) {
        const hora = data.split(' ')[1];
        const dataAmericana = data.split(' ')[0].split('/').reverse().join('-');
        const dataFormatada = `${dataAmericana} ${hora}`;

        return dataFormatada;
    }

    public filtrarCalendario() {
        this.filtroHabilitado = true;
        const periodo = Metadata.getInstance<DateRange>('data').value;
        const calendario = Metadata.getInstance<Calendar>('calendarioFmapp');
        const dataInicial = periodo[0];
        const dataFinal = (periodo.length > 1) ? periodo[1] : null;
        const local = Metadata.getInstance<Select>('filtro-local').value;
        const localSup = Metadata.getInstance<Select>('filtro-local-superior').value;
        const tipoLocal = Metadata.getInstance<Select>('filtro-tipoLocal').value;
        const operador = Metadata.getInstance<Select>('filtro-operador').value;
        const equipes = Metadata.getInstance<Select>('filtro-equipes').value;
        const clientes = Metadata.getInstance<Select>('filtro-clientes').value;
        const form = Metadata.getInstance<Form>('form-filtros');

        if (Utilities.validateForm(form)) {
            this.postTarefasCalendario({ component: calendario }, dataInicial, dataFinal, local, localSup, tipoLocal, operador, equipes, clientes);
        }
    }

    public clickEvento({ component }: any) {
        if (this.filtroHabilitado !== true) {
            const [ano, mes] = component.value.split('-');
            const dataInicial = new Date(ano, mes - 1, 1);
            const dataFinal = new Date(ano, mes, 0);
            this.postTarefasCalendario({ component }, dataInicial.toLocaleDateString(), dataFinal.toLocaleDateString(), null, null, null, null, null, null);
        }
    }
}
